﻿using System;
using System.Web.UI;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.Routing;
using ThinkGeo.MapSuite.WebEdition;

namespace HowDoISamples
{
    public partial class SimpleRouting : System.Web.UI.Page
    {
        private static RoutingEngine routingEngine;
        private static RoutingSource RoutingSourceForShortest;
        private static RoutingSource RoutingSourceForFastest;

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                Map1.MapUnit = GeographyUnit.DecimalDegree;
                Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
                Map1.CurrentExtent = new RectangleShape(-97.763384, 30.299707, -97.712382, 30.259309);

                WorldMapKitWmsWebOverlay worldMapKitOverlay = new WorldMapKitWmsWebOverlay();
                Map1.BackgroundOverlay = worldMapKitOverlay;

                ShapeFileFeatureSource featureSource = new ShapeFileFeatureSource(Server.MapPath(@"SampleData\Austinstreets.shp"));
                featureSource.Open();
                RoutingLayer routingLayer = new RoutingLayer();
                //routingLayer.StartPoint = featureSource.GetFeatureById(txtStartFeatureId.Value, ReturningColumnsType.NoColumns).GetShape().GetCenterPoint();
                //routingLayer.EndPoint = featureSource.GetFeatureById(txtEndFeatureId.Value, ReturningColumnsType.NoColumns).GetShape().GetCenterPoint();
                routingLayer.RouteStyle.OuterPen.Color = GeoColor.SimpleColors.Blue;
                Map1.DynamicOverlay.Layers.Add("RoutingLayer", routingLayer);
                Map1.DynamicOverlay.Opacity = 0.4F;

                PointShape startPoint = featureSource.GetFeatureById(txtStartFeatureId.Value, ReturningColumnsType.NoColumns).GetShape().GetCenterPoint();
                PointShape endPoint = featureSource.GetFeatureById(txtEndFeatureId.Value, ReturningColumnsType.NoColumns).GetShape().GetCenterPoint();

                SimpleMarkerOverlay markerOverlay = new SimpleMarkerOverlay("MarkerOverlay");

                markerOverlay.Markers.Add(new Marker(startPoint.X, startPoint.Y, new WebImage("Icons/StartpointMarker.png", 17, 25, -12.5f, -17f)));
                markerOverlay.Markers.Add(new Marker(endPoint.X, endPoint.Y, new WebImage("Icons/EndpointMarker.png", 17, 25, -12.5f, -17f)));
                Map1.CustomOverlays.Add(markerOverlay);

                RoutingSourceForShortest = new RtgRoutingSource(Server.MapPath(@"SampleData\Austinstreets.rtg"));
                RoutingSourceForFastest = new RtgRoutingSource(Server.MapPath(@"SampleData\AustinStreetsForFastest.rtg"));
                routingEngine = new RoutingEngine(RoutingSourceForShortest, featureSource);

                FindPath();
            }
        }

        private void FindPath()
        {
            RoutingLayer routingLayer = (RoutingLayer)Map1.DynamicOverlay.Layers["RoutingLayer"];

            routingLayer.Routes.Clear();
            routingLayer.Routes.Add(routingEngine.GetRoute(txtStartFeatureId.Value, txtEndFeatureId.Value).Route);
            Map1.DynamicOverlay.Redraw();
        }

        protected void rbtShortest_CheckedChanged(object sender, EventArgs e)
        {
            routingEngine.RoutingSource = RoutingSourceForShortest;
            FindPath();
        }

        protected void rbtFastest_CheckedChanged(object sender, EventArgs e)
        {
            routingEngine.RoutingSource = RoutingSourceForFastest;
            FindPath();
        }
    }
}
